"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vite_plugin_svelte_1 = require("@sveltejs/vite-plugin-svelte");
const path_1 = __importDefault(require("path"));
const analyze_component_1 = require("./analyze-component");
const svelte_reader_1 = require("./svelte-reader");
const svelteFrameworkPlugin = {
    isCompatible: async (dependencies) => {
        var _a;
        const version = await ((_a = dependencies["svelte"]) === null || _a === void 0 ? void 0 : _a.readInstalledVersion());
        if (!version) {
            return false;
        }
        return parseInt(version) === 3;
    },
    async create() {
        const previewDirPath = path_1.default.resolve(__dirname, "..", "preview");
        return {
            pluginApiVersion: 3,
            name: "@previewjs/plugin-svelte",
            transformReader: (reader) => (0, svelte_reader_1.createSvelteTypeScriptReader)(reader),
            defaultWrapperPath: "__previewjs__/Wrapper.svelte",
            previewDirPath,
            detectComponents: async (typeAnalyzer, absoluteFilePaths) => {
                const components = [];
                for (const absoluteFilePath of absoluteFilePaths) {
                    if (absoluteFilePath.endsWith(".svelte")) {
                        const name = path_1.default.basename(absoluteFilePath, path_1.default.extname(absoluteFilePath));
                        components.push({
                            absoluteFilePath,
                            name,
                            offsets: [[0, Infinity]],
                            info: {
                                kind: "component",
                                exported: true,
                                analyze: async () => (0, analyze_component_1.analyzeSvelteComponent)(typeAnalyzer, absoluteFilePath + ".ts"),
                            },
                        });
                    }
                    // TODO: Storybook support.
                }
                return components;
            },
            viteConfig: () => ({
                define: {
                    __SVELTEKIT_DEV__: "false",
                },
                publicDir: "static",
                resolve: {
                    alias: {
                        $app: ".svelte-kit/runtime/app",
                    },
                },
                plugins: [
                    (0, vite_plugin_svelte_1.svelte)(),
                    {
                        name: "previewjs:disable-svelte-hmr",
                        async transform(code, id) {
                            if (!id.endsWith(".svelte")) {
                                return null;
                            }
                            return code.replace(/import\.meta/g, "({})");
                        },
                    },
                ],
            }),
            incompatibleVitePlugins: ["vite-plugin-svelte-kit"],
        };
    },
};
exports.default = svelteFrameworkPlugin;
