"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const type_analyzer_1 = require("@previewjs/type-analyzer");
const vfs_1 = require("@previewjs/vfs");
const path_1 = __importDefault(require("path"));
const vitest_1 = require("vitest");
const svelte_reader_1 = require("./svelte-reader");
(0, vitest_1.describe)("createSvelteTypeScriptReader", () => {
    let memoryReader;
    let reader;
    let typeAnalyzer;
    (0, vitest_1.beforeEach)(() => {
        memoryReader = (0, vfs_1.createMemoryReader)();
        reader = (0, svelte_reader_1.createSvelteTypeScriptReader)(memoryReader);
        typeAnalyzer = (0, type_analyzer_1.createTypeAnalyzer)({
            rootDirPath: path_1.default.join(__dirname, "virtual"),
            reader: (0, vfs_1.createStackedReader)([
                memoryReader,
                (0, vfs_1.createFileSystemReader)(), // required for TypeScript libs, e.g. Promise
            ]),
            specialTypes: {},
        });
    });
    (0, vitest_1.afterEach)(() => {
        typeAnalyzer.dispose();
    });
    (0, vitest_1.it)("extracts from script", async () => {
        memoryReader.updateFile(path_1.default.join(__dirname, "virtual", "App.svelte"), `
<script lang="ts">
  import logo from './assets/svelte.png'
  import Counter from './lib/Counter.svelte'
</script>

<main>
  <img src={logo} alt="Svelte Logo" />
  <h1>Hello Typescript!</h1>
  <Counter />
</main>   
    `);
        const virtualFile = await reader.read(path_1.default.join(__dirname, "virtual", "App.svelte.ts"));
        if ((virtualFile === null || virtualFile === void 0 ? void 0 : virtualFile.kind) !== "file") {
            throw new Error();
        }
        (0, vitest_1.expect)(await virtualFile.read()).toEqual(`
  import logo from './assets/svelte.png'
  import Counter from './lib/Counter.svelte'
`);
    });
    // TODO: Add a test to ensure that module scripts are ignored.
});
